@@ -11,7 +11,7 @@ import cmd
 
 import ascii_magic
 
-from weconnect import weconnect, addressable, errors
+from weconnect import weconnect, addressable, errors, util, domain
 from weconnect.__version import __version__ as __weconnect_version__
 
 from weconnect_cli.__version import __version__
@@ -54,13 +54,12 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
         description='Commandline Interface to interact with the Volkswagen WeConnect Services')
     parser.add_argument('--version', action='version',
                         version=f'%(prog)s {__version__} (using WeConnect-python {__weconnect_version__})')
-    parser.add_argument('-u', '--username', help='Username of Volkswagen id', required=False)
-    parser.add_argument('-p', '--password', help='Password of Volkswagen id', required=False)
+    weConnectGroup = parser.add_argument_group('WeConnect')
+    weConnectGroup.add_argument('-u', '--username', help='Username of Volkswagen id', required=False)
+    weConnectGroup.add_argument('-p', '--password', help='Password of Volkswagen id', required=False)
     defaultNetRc = os.path.join(os.path.expanduser("~"), ".netrc")
-    parser.add_argument('--netrc', help=f'File in netrc syntax providing login (default: {defaultNetRc}).'
-                        ' Netrc is only used when username and password are not provided  as arguments',
-                        default=None, required=False)
-    parser.add_argument('-v', '--verbose', action="append_const", const=-1,)
+    weConnectGroup.add_argument('--netrc', help=f'File in netrc syntax providing login (default: {defaultNetRc}).'
+                                ' Netrc is only used when username and password are not provided  as arguments', default=None, required=False)
     parser.add_argument('--no-token-storage', dest='noTokenStorage', help='Do not store token on filesystem (this'
                         ' will cause a new login for every invokation!)', action='store_true')
     defaultTemp = os.path.join(tempfile.gettempdir(), 'weconnect.token')
@@ -69,18 +68,28 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
     defaultCacheTemp = os.path.join(tempfile.gettempdir(), 'weconnect.cache')
     parser.add_argument('--cachefile', help=f'file to store cache (default: {defaultCacheTemp})',
                         default=defaultCacheTemp)
-    parser.add_argument('-i', '--interval', help='Query interval in seconds, used for cache and events',
-                              type=NumberRangeArgument(1), required=False, default=300)
-    parser.add_argument('--picture-cache-interval', dest='pictureCache', help='Picture download interval in seconds, this does not influence the interval in'
-                        ' which the status picture is updated', type=NumberRangeArgument(1), required=False, default=86400)
-    parser.add_argument('-l', '--chargingLocation', nargs=2, metavar=('latitude', 'longitude'), type=float,
-                        help='If set charging locations will be added to the result around the given coordinates')
-    parser.add_argument('--chargingLocationRadius', type=NumberRangeArgument(0, 100000),
-                        help='Radius in meters around the chargingLocation to search for chargers')
-    parser.add_argument('--no-capabilities', dest='noCapabilities', help='Do not add capabilities', action='store_true')
-    parser.add_argument('--no-pictures', dest='noPictures', help='Do not add pictures', action='store_true')
+    weConnectGroup.add_argument('-i', '--interval', help='Query interval in seconds, used for cache and events',
+                                type=NumberRangeArgument(1), required=False, default=300)
+    weConnectGroup.add_argument('--picture-cache-interval', dest='pictureCache', help='Picture download interval in seconds, this does not influence the'
+                                ' interval in which the status picture is updated', type=NumberRangeArgument(1), required=False, default=86400)
+    weConnectGroup.add_argument('-l', '--chargingLocation', nargs=2, metavar=('latitude', 'longitude'), type=float,
+                                help='If set charging locations will be added to the result around the given coordinates')
+    weConnectGroup.add_argument('--chargingLocationRadius', type=NumberRangeArgument(0, 100000),
+                                help='Radius in meters around the chargingLocation to search for chargers')
+    weConnectGroup.add_argument('--no-capabilities', dest='noCapabilities', help='Do not add capabilities', action='store_true')
+    weConnectGroup.add_argument('--no-pictures', dest='noPictures', help='Do not add pictures', action='store_true')
+    weConnectGroup.add_argument('--selective', help='Just fetch status of a certain type', default=None, required=False, action='append',
+                                type=domain.Domain, choices=list(domain.Domain))
     parser.add_argument('--elapsed-statistics', dest='elapsedStatistics', help='Statistics over server response times', action='store_true')
 
+    loggingGroup = parser.add_argument_group('Logging')
+    loggingGroup.add_argument('-v', '--verbose', action="append_const", help='Logging level (verbosity)', const=-1,)
+    loggingGroup.add_argument('--logging-format', dest='loggingFormat', help='Logging format configured for python logging '
+                              '(default: %%(asctime)s:%%(module)s:%%(message)s)', default='%(asctime)s:%(levelname)s:%(message)s')
+    loggingGroup.add_argument('--logging-date-format', dest='loggingDateFormat', help='Logging format configured for python logging '
+                              '(default: %%Y-%%m-%%dT%%H:%%M:%%S%%z)', default='%Y-%m-%dT%H:%M:%S%z')
+    loggingGroup.add_argument('--hide-repeated-log', dest='hideRepeatedLog', help='Hide repeated log messages from the same module', action='store_true')
+
     parser.set_defaults(command='shell')
 
     subparsers = parser.add_subparsers(title='commands', description='Valid commands',
@@ -111,7 +120,10 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
     for adjustment in args.verbose or ():
         logLevel = min(len(LOG_LEVELS) - 1, max(logLevel + adjustment, 0))
 
-    logging.basicConfig(level=LOG_LEVELS[logLevel])
+    logging.basicConfig(level=LOG_LEVELS[logLevel], format=args.loggingFormat, datefmt=args.loggingDateFormat)
+    if args.hideRepeatedLog:
+        for handler in logging.root.handlers:
+            handler.addFilter(util.DuplicateFilter())
 
     ascii_magic.init_terminal()
 
@@ -150,7 +162,7 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
     try:  # pylint: disable=too-many-nested-blocks
         weConnect = weconnect.WeConnect(username=username, password=password, tokenfile=tokenfile,
                                         updateAfterLogin=False, loginOnInit=False, updateCapabilities=(not args.noCapabilities),
-                                        updatePictures=(not args.noPictures))
+                                        updatePictures=(not args.noPictures), selective=args.selective)
         if args.noCache or not os.path.isfile(args.cachefile):
             weConnect.login()
         else:
@@ -170,7 +182,7 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
 
         if args.command == 'shell':
             try:
-                weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures))
+                weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures), selective=args.selective)
                 # disable caching
                 weConnect.maxAge = None
                 weConnect.clearCache()
@@ -178,7 +190,7 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
             except KeyboardInterrupt:
                 pass
         elif args.command == 'list':
-            weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures))
+            weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures), selective=args.selective)
             allElements = weConnect.getLeafChildren()
             for element in allElements:
                 if args.setters:
@@ -187,7 +199,7 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
                 else:
                     print(element.getGlobalAddress())
         elif args.command == 'get':
-            weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures))
+            weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures), selective=args.selective)
             element = weConnect.getByAddressString(args.id)
             if element:
                 if isinstance(element, dict):
@@ -198,7 +210,7 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
                 print(f'id {args.id} not found', file=sys.stderr)
                 sys.exit(-1)
         elif args.command == 'set':
-            weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures))
+            weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures), selective=args.selective)
             element = weConnect.getByAddressString(args.id)
             if element:
                 try:
@@ -211,13 +223,13 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
                           file=sys.stderr)
                     sys.exit(-1)
                 except errors.SetterError as err:
-                    print(f'id {args.id} cannot be set: %s', err, file=sys.stderr)
+                    print(f'id {args.id} cannot be set: {err}', file=sys.stderr)
                     sys.exit(-1)
             else:
                 print(f'id {args.id} not found', file=sys.stderr)
                 sys.exit(-1)
         elif args.command == 'save':
-            weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures))
+            weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures), selective=args.selective)
             element = weConnect.getByAddressString(args.id)
             if element:
                 try:
@@ -252,7 +264,7 @@ def main():  # noqa: C901 # pylint: disable=too-many-statements,too-many-branche
             weConnect.addObserver(observer, addressable.AddressableLeaf.ObserverEvent.VALUE_CHANGED,
                                   priority=addressable.AddressableLeaf.ObserverPriority.USER_MID)
             while True:
-                weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures))
+                weConnect.update(updateCapabilities=(not args.noCapabilities), updatePictures=(not args.noPictures), selective=args.selective)
                 time.sleep(args.interval)
         else:
             LOG.error('command not implemented')
